/**
 * CMI : Cluster Method Invocation
 * Copyright (C) 2007-2008 Bull S.A.S.
 * Contact: cmi@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * --------------------------------------------------------------------------
 * $Id: RemoteIIOPCMIReferenceableWrapper.java 2183 2008-10-19 13:20:42Z loris $
 * --------------------------------------------------------------------------
 */

package org.ow2.cmi.reference.serialization;

import java.io.InvalidObjectException;
import java.io.ObjectStreamException;
import java.rmi.Remote;

import javax.naming.NamingException;

import org.ow2.cmi.reference.CMIReference;
import org.ow2.cmi.reference.CMIReferenceableWrapper;

/**
 * Wrap an Interoperable Object Reference with the CMI reference on its associated remote object.
 * @author Loris Bouzonnet
 */
public class RemoteIIOPCMIReferenceableWrapper<T extends Remote> extends CMIReferenceableWrapper<T> {

    /**
     *  Id for serializable class.
     */
    private static final long serialVersionUID = 1192398491336031370L;

    /**
     * Interoperable Object Reference.
     */
    private String ior;


    public RemoteIIOPCMIReferenceableWrapper(final CMIReference cmiRef, final T referencedObject) {
        super(cmiRef, referencedObject);
    }

    public RemoteIIOPCMIReferenceableWrapper(final CMIReference cmiRef, final String ior) {
        super(cmiRef, null);
        this.ior = ior;
    }

    @SuppressWarnings("unchecked")
    private Object readResolve() throws ObjectStreamException {
        CMIReference ref = getReference();
        T remote;
        try {
            remote = (T) Utility.string_to_remote(ior);
        } catch (NamingException e) {
            throw new InvalidObjectException("Cannot get the stub from the ior " + ior);
        }
        // Replace the IOR by the corresponding stub IIOP
        return new RemoteIIOPCMIReferenceableWrapper<T>(ref, remote);
    }

    private Object writeReplace() throws ObjectStreamException {
        CMIReference ref = getReference();
        String ior;
        try {
            ior = Utility.remote_to_string(getReferencedObject());
        } catch (NamingException e) {
            throw new InvalidObjectException("Cannot get the ior from the stub " + getReferencedObject());
        }
        // Replace the stub IIOP by its IOR
        return new RemoteIIOPCMIReferenceableWrapper<T>(ref, ior);

    }

    @Override
    public boolean equals(final Object obj) {
        return super.equals(obj);
    }

    @Override
    public int hashCode() {
        return super.hashCode();
    }

}
